/*******************************************************************************

    TERMO_AQUARIO


    TERMOSTATO C/ PIC12F675 E LCD (MTODO LPLEX - 3 FIOS)


    AUTOR: CLAUDIO LARIOS

    DATA:13/06/2016
    
    FAIXA DE AJUSTE: ENTRE 15 A 40 GRAUS CENTIGRADOS
    
    (SALVA AJUSTE NA EEPROM DO PIC)

    APLICAO: CONTROLE DE TEMPERATURA EM AQURIOS 

    IMPORTANTE: USE CAPACITOR DE 220nF JUNTO AOS PINOS 1 E 8 DO PIC E NO 
    1 E 2 DO LCD, POIS O CIRCUITO  SENSVEL A INTERFERENCIAS PELA ALIMENTAO

    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>//pic usado
#device adc=10
#use delay(clock=4000000)//frequencia de clock
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT//configurao
#use fast_io(a)//modo rpido
#rom 0x2100 = {0xc8,0} //20 graus
//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  t1con  = 0x10

//pinos do lcd (metodo Lariosplex - 3 fios)
#bit DB76  = 0x5.0  //pino 7 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54  = 0x5.5  //pino 2 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit E_RS  = 0x5.2  //pino 5 E=direto RS=resistor 10k +cap 10nF ao gnd
#bit sw    = 0x05.3  //pino 4 sw - ajuste de temperatura
#bit rele  = 0x05.1 // pino 6 rele de liga/desliga resistor aquecedor do aqurio

#bit  TRIS_DB54 = 0X85.1 //sentido do pino 6

//DEFINE DO LCD LARIOSPLEX
#define VCARGA 500 // por tentativa e erro (entre 200 a 500)
#define VDESCARGA 500//idem
#define V_DELAY_CLK 2  // de 2 a 50 conforme LCD

//histerese e filtro anti-oscilao
#define FATOR_HISTERESE  5  //0,5 grau centigrado de histerese no liga/desliga
#define VALOR_FILTRO_ANTI_OSC 5 // filtragem para evitar oscilao na medio

//defines do controle max e minimo de temperatura
 #define LIMITE_ESCALA_SUPERIOR 400  //40.0 graus como limite superior
 #define LIMITE_ESCALA_INFERIOR 150  //15.0 graus como limite inferior
 
 //define
#define V_TEMPO_INVERTE 7 //tempo para liberar a inverso do boto de ajuste
#define VELOCIDADE_DO_AJUSTE 100 // velocidade do ajuste em mseg

//variveis globais
unsigned int1 RS,f_sw,f_inv;
unsigned int8 cent,dez,un,a,muda,cont_t;
unsigned int16 lido,res,res_sensor, res_adj,res_atual;
//unsigned int32 soma;

/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/


void delay1seg(){delay_ms(1000);}//delay 1seg
void delay_4ms(){delay_ms(4);}//delay 4ms
void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(VDESCARGA);}//descarga do cap
void t_carga(){delay_us(VCARGA);}//tempo para carga do cap
void clk(){ E_RS=1; delay_cycles(V_DELAY_CLK); E_RS=0;}//gera clock par Lcd


/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();//tempo para carregar capacitor
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock

}

/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/

void env_lcd_d(unsigned int8 dado){

 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);//envia low nibble
 delay_4ms();//delay
}

/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){

 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);//low nibble
 delay_4ms();//delay
}

/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/

void Lcd_Init(){
  unsigned int8 a;

   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(40);//delay inicial
   for(a=0;a<3;a++) { env_nible(3);delay_4ms();}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_4ms();//delay 4 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
   env_lcd_c(0x01);//limpa tela
}

/*******************************************************************************
               MOSTRA NO LCD A LEITURA PROCESSADA
*******************************************************************************/
void mostra_res(){
  unsigned int1 zesq=0;
     if(!cent&&!zesq){cent=240;}else{ zesq=1;}//apaga zeros a esquerda
     env_lcd_d(cent+'0');//envia centena
     env_lcd_d(dez+'0');//envia dezena
     env_lcd_d(',');//envia vrgula
     env_lcd_d(un+'0');//envia unidade
  }
  
/*******************************************************************************
                 CONVERTE DE HEXA PARA DECIMAL
*******************************************************************************/

void converte_res(){
    cent=0;dez=0;un=0;
    while(res>=100){res-=100; cent++;}
    while(res>=10){res-=10; dez++;}
    un=res;
}

/*******************************************************************************
                   MOSTRA TEMPERATURA NO LCD
*******************************************************************************/
void Le_Temp(){
   TRIS_DB54=1;//entrada de vref para o adc
   delay_ms(50);//aguarda
   lido=READ_ADC();// l valor da tenso de entrada do pino 5
     for(a=0;a<49;a++){//soma as 50 leituras sucessivas
     lido+=READ_ADC();
     }
     TRIS_DB54=0;//volta a ser sada
     lido/=50;//mdia das ultimas 50 leituras
     res_atual=lido;
     //filtro por software para evitar oscilao no digito ps virgula
     if(res_sensor!=res_atual){
       muda++;
       if(muda>=VALOR_FILTRO_ANTI_OSC) {muda=0; res=res_atual;}
       else{ res=res_sensor; }
      }
     else{ res=res_sensor; muda=0; }
     res_sensor=res;//salva leitura para posterior comparao
     env_lcd_c(0x80); // 1 posico da linha 1
     printf(env_lcd_d(" Termo_AQuario  "));//mostra o nome
     env_lcd_c(0xc0); // 1 posico da linha 2
     printf(env_lcd_d("T="));//envia para lcd
     converte_res();// ajusta para decimal
     mostra_res(); //mostra resultado no LCD
     printf(env_lcd_d("C"));// mostra nome da unidade de medida (C)
     lido=read_eeprom(0);
     lido+=(int16)read_eeprom(1)<<8;
     res=lido;
     res_adj=res;//salva para futura comparao
     printf(env_lcd_d(" A="));//envia para lcd
     converte_res();// ajusta para decimal
     mostra_res(); //mostra resultado no LCD
     printf(env_lcd_d("C "));// mostra nome da unidade de medida (C)
}

/*******************************************************************************
       COMPARA TEMPERATURA DO SENSOR LM35 COM A TEMPERATURA DESEJADA
*******************************************************************************/
void Compara_Temp(){
  if(res_sensor>=res_adj) rele=0;
  if((res_sensor+FATOR_HISTERESE)<=res_adj)rele=1;
}
/*******************************************************************************
       ROTINA DE ESCRITA NA EEPROM INTERNA DO PIC
*******************************************************************************/

void escreve_eep(unsigned int8 address,unsigned int8 val){
   write_eeprom(address,val);
}

/*******************************************************************************
       AJUSTA A TEMPERATURA DESEJADA
*******************************************************************************/

void Adj_Temp(){
 unsigned int16 d;
 
  if(sw)  return;//se interruptor no foi pressionado sai desta rotina
   f_sw=1;//seta flag indicador de boto apertado
   cont_t= V_TEMPO_INVERTE;//carga de tempo para inverso do ajuste
   d=read_eeprom(0);//recupera valor do ajuste que foi salvo na eeprom
   d+=(int16) read_eeprom(1)<<8;
   
   if(!f_inv){//testa flag de inverso
   d++;//incrementa valores
   if(d>=LIMITE_ESCALA_SUPERIOR){ d=LIMITE_ESCALA_INFERIOR; }//limita a excurso da escala   
  // escreve_eep(0,d); escreve_eep(1,d>>8);
   }
   else{
   d--;//decrementa valores
   if(d<=LIMITE_ESCALA_INFERIOR){ d=LIMITE_ESCALA_SUPERIOR;  }      
   }
    escreve_eep(0,d); escreve_eep(1,d>>8);  //salva novo valor na eeprom
   
   delay_ms(VELOCIDADE_DO_AJUSTE);//delay de velocidade de incremento
  }
/*******************************************************************************
      INVERTE SENTIDO DO AJUSTE (INCREMENTOS/DECREMENTO)
*******************************************************************************/
void Inv_adj(){
    if(f_sw) {//se flag estiver setado
       if(--cont_t==0) {f_inv=!f_inv; f_sw=0;}//contagem em zero faz inverso
     }// 
}

/*******************************************************************************
                        ROTINA MAIN
*******************************************************************************/

void main() {

   
   setup_adc_ports(sAN3 | VSS_VREF);  //seleciona AN2 e pino 6 como referncia
   setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   set_adc_channel(3); //ajusta para o pino 3 (AN3 - gp4)
   setup_comparator(NC_NC_NC_NC);//desliga comparadores
   setup_vref(FALSE);//desliga vref
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);//desliga timer 1
   trisio=0b011000;//ajusta entradas e sadas
   gpio=0;//zera tudo
   descarregar();//descarga dos capacitores do Lariosplex
   Lcd_Init();//inicializa Lcd
   printf(env_lcd_d(" Termo_AQuario  "));//mostra o nome
   env_lcd_c(0xc0);//linha 2
   printf(env_lcd_d("Larios.Tecnol.Ws"));// o blog
   delay1seg();//delay de 1 segundo
   delay1seg();//idem
   res=0;//condio inicial do regs e flags
   f_inv=0;
   f_sw=0;

   /*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


   for(;;){
/*******************************************************************************
              ROTINA DE OPERAO DO TERMOSTATO
*******************************************************************************/
     Le_Temp();//le temperatura do aqurio
     Compara_Temp();//compara valor gravado na eeprom com temperatura atual
     Adj_Temp(); //testa chave para regular temperatura desejada
     Inv_adj();//inverte sentido de incremento/decremento aps boto ser solto    
     delay_ms(100);// limita o nmero de apresentaes do lcd por segundo
    
   }//for(;;)

}//void main()
